﻿/**
* Main game class for episode 28 of One of those Weeks.
* Holds all the flags and data so different scenes all have access to all necessary information.
*
* @author Billy D. Spelchan
* @version 1.0
*/
class OotW28
{
	// ***** CONSTANTS *****
	public static var MESSAGEHIDE:Number = 1;

	public static var TILESET:String = "-abcdehmn";
	public static var TILE_EMPTY:Number = 0;
	public static var TILE_TRIANGLE:Number = 1;
	public static var TILE_SQUARE:Number = 2;
	public static var TILE_PENTAGRAM:Number = 3;
	public static var TILE_HEXAGON:Number = 4;
	public static var TILE_OCTAGON:Number = 5;
	public static var TILE_NOTE:Number = 6;
	public static var TILE_DOOR:Number = 7;
	public static var TILE_WALL:Number = 8;

	public static var COIN_NAMES = new Array(
			"Triangle", "Square", "Pentagram", "Hexagon", "Octagon"
	);
	public static var PLAYER_STARTX = 15;
	public static var PLAYER_STARTY = 16;
	
	// ***** VARIABLES *****
	
	public var haveCoins:Array = new Array(5);
	
	public var RAW_MAP:Array;
	public var dirString:Array;
	public var tdTilemap:Array;
	public var automapData:Array;
	public var playerX:Number = PLAYER_STARTX;
	public var playerY:Number = PLAYER_STARTY;
	public var playerD:Number = 0;
	public var puzzleSlotState:Array = new Array(5);
	
	// ***** CONSTRUCTOR *****
	public function OotW28()
	{
		RAW_MAP = new Array(
//1234567890123456789012345678901
"nnnnnnnnnnnnnnnmnnnnnnnnnnnnnnnn",
"nb--n---nnn-------n-----------an",
"nnn-n-n--n--nnnn-nnnnn-nnnnnnnnn",
"n---nnn-nn-nn-----n--n--n------n",
"n-nnn---n--n--n-n-n-nnn-n-nnnn-n",
"n-n-n-nnn-nn-nn---n-n-n---n----n",
"n---n--n--n---nnnnn-n-n-n-n-n-nn",
"n-nnnn-nn---n----n--n-n-nnnnn--n",
"n-------nn--nnnn-n-nn-n-n---nnnn",
"nnn-nn---nnnn------n--n-n-n--n-n",
"n-n-n--n----nn-nnnnn-nn-n-nnnn-n",
"n-nnn-nnnnn--n--nn---n--n------n",
"n-n-nnn---nn-nn----nnn-nnnnnnn-n",
"n-n---nnn-----n-n-nn-----------n",
"n-nn-nn-nnn-n-n-n-nnn--nnnnnnnnn",
"n-n--n----nnn-nhnnn---nn-------n",
"n-n-nn-nnnn---------nnn--nnnnn-n",
"n-n--n-n----n-n-nnnnnn--nn---n-n",
"n-nn-n-n-nnnn-n-n------nn--n-n-n",
"n----n---n-n--n---nnnnnnnnnn---n",
"nnn-nn-nnn-n-nnnn------n---nnnnn",
"n----n--n-------nnnnnn---n-----n",
"n-nnnnn-nnn-n-n---n--nn-nnnnnn-n",
"n--------n--nnnnn-n---n------n-n",
"n-nnnnnn-nn-------n-n-nnn-n-nn-n",
"n-n-------nn-nnnnnn-n-n-n-n--n-n",
"n-n-nnnnn--n--------n---n-nn-n-n",
"n-nnn---nn-nnnn-nnnnnnn-n--n-n-n",
"n---n-n-n--n----n-------nn-n-n-n",
"nnn-n-nnn-nn-nnnn-nnnnn-nnnn-n-n",
"nd--n-----n-------ne------n--ncn",
"nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn"
//1234567890123456789012345678901
);
		dirString = new Array( "North", "East", "South", "West");
		tdTilemap = new Array(32);
		automapData = new Array;
		for (var cntr:Number = 0; cntr < 32; ++cntr)
			automapData[cntr] = new Array(32);
	}
	
	public function restartGame()
	{
		var cntr:Number;
		
		playerX = PLAYER_STARTX;
		playerY = PLAYER_STARTY;
		playerD = 0;
		for (cntr = 0; cntr < 32; ++cntr)
			tdTilemap[cntr] = RAW_MAP[cntr];
		for (var cntrY:Number = 0; cntrY < 32; ++cntrY)
			for (var cntrX:Number = 0; cntrX < 32; ++cntrX)
				automapData[cntrY][cntrX] = 0;
		for (cntr = 0; cntr < 5; ++cntr) {
			haveCoins[cntr] = false;
			puzzleSlotState[cntr] = false;
		}
	}
	
	function getOffsetTile(dist:Number, lr:Number):Number
	{
		var tx:Number = playerX;
		var ty:Number = playerY;
		var lradj:Number = lr;
		switch (playerD)
		{
			case 0: // north
				tx += lradj;
				ty -= dist;
				break;
			case 1: // east
				tx += dist;
				ty += lradj;
				break;
			case 2: // south
				tx -= lradj;
				ty += dist;
				break;
			case 3: // west
				tx -= dist;
				ty -= lradj;
				break;
		}
		while(tx < 0) tx += 32;
		while(tx > 31) tx -= 32;
		while (ty < 0) ty += 32;
		if (ty > 31)
			return TILE_WALL;
	//	while (ty > 31) ty -= 32;
		var tchar:String = tdTilemap[ty].charAt(tx);
		var tval:Number = TILESET.indexOf(tchar);
		var coinIndex = tval - TILE_TRIANGLE
		if ( (tval >= TILE_TRIANGLE) && (tval <= TILE_OCTAGON))
			if ((haveCoins[coinIndex] == true) ||
					(puzzleSlotState[coinIndex] == true))
				tval = TILE_EMPTY;
		return tval;
	}

	function canMoveForward():Boolean
	{
		return (getOffsetTile(1,0) < TILE_DOOR);
	}
	
	function enterTile(nx:Number, ny:Number):Number
	{		
		var id:Number = getOffsetTile(0,0);
		var tempTile:Number;
		
		var tx:Number, ty:Number;
		var tchar:String;
		for (var cntrX:Number = -1; cntrX < 2; ++cntrX) {
			tx = nx+cntrX;
			if (tx < 0) continue;
			if (tx > 31) break;
			for (var cntrY:Number = -1; cntrY < 2; ++cntrY) {
				ty = (ny+cntrY);
				if (ty < 0) continue;
				if (ty > 31) break;
				tchar = tdTilemap[ty].charAt(tx);
				tempTile = TILESET.indexOf(tchar);
				automapData[ny+cntrY][nx+cntrX] = tempTile + 1; // remember 0 is used for unseen tile
			}
		}
		return id;
	}
	
}